@@ -59,12 +59,20 @@ static gboolean
 gst_audio_downmix_meta_transform (GstBuffer * dest, GstMeta * meta,
     GstBuffer * buffer, GQuark type, gpointer data)
 {
-  GstAudioDownmixMeta *smeta;
+  GstAudioDownmixMeta *smeta, *dmeta;
 
   smeta = (GstAudioDownmixMeta *) meta;
-  gst_buffer_add_audio_downmix_meta (dest, smeta->from_position,
-      smeta->from_channels, smeta->to_position, smeta->to_channels,
-      (const gfloat **) smeta->matrix);
+
+  if (GST_META_TRANSFORM_IS_COPY (type)) {
+    dmeta = gst_buffer_add_audio_downmix_meta (dest, smeta->from_position,
+        smeta->from_channels, smeta->to_position, smeta->to_channels,
+        (const gfloat **) smeta->matrix);
+    if (!dmeta)
+      return FALSE;
+  } else {
+    /* return FALSE, if transform type is not supported */
+    return FALSE;
+  }
 
   return TRUE;
 }
@@ -181,13 +189,117 @@ gst_audio_downmix_meta_get_info (void)
 {
   static const GstMetaInfo *audio_downmix_meta_info = NULL;
 
-  if (g_once_init_enter (&audio_downmix_meta_info)) {
+  if (g_once_init_enter ((GstMetaInfo **) & audio_downmix_meta_info)) {
     const GstMetaInfo *meta =
         gst_meta_register (GST_AUDIO_DOWNMIX_META_API_TYPE,
         "GstAudioDownmixMeta", sizeof (GstAudioDownmixMeta),
         gst_audio_downmix_meta_init, gst_audio_downmix_meta_free,
         gst_audio_downmix_meta_transform);
-    g_once_init_leave (&audio_downmix_meta_info, meta);
+    g_once_init_leave ((GstMetaInfo **) & audio_downmix_meta_info,
+        (GstMetaInfo *) meta);
   }
   return audio_downmix_meta_info;
 }
+
+static gboolean
+gst_audio_clipping_meta_init (GstMeta * meta, gpointer params,
+    GstBuffer * buffer)
+{
+  GstAudioClippingMeta *cmeta = (GstAudioClippingMeta *) meta;
+
+  cmeta->format = GST_FORMAT_UNDEFINED;
+  cmeta->start = cmeta->end = 0;
+
+  return TRUE;
+}
+
+static gboolean
+gst_audio_clipping_meta_transform (GstBuffer * dest, GstMeta * meta,
+    GstBuffer * buffer, GQuark type, gpointer data)
+{
+  GstAudioClippingMeta *smeta, *dmeta;
+
+  smeta = (GstAudioClippingMeta *) meta;
+
+  if (GST_META_TRANSFORM_IS_COPY (type)) {
+    GstMetaTransformCopy *copy = data;
+
+    if (copy->region)
+      return FALSE;
+
+    dmeta =
+        gst_buffer_add_audio_clipping_meta (dest, smeta->format, smeta->start,
+        smeta->end);
+    if (!dmeta)
+      return FALSE;
+  } else {
+    /* TODO: Could implement an automatic transform for resampling */
+    /* return FALSE, if transform type is not supported */
+    return FALSE;
+  }
+
+  return TRUE;
+}
+
+/**
+ * gst_buffer_add_audio_clipping_meta:
+ * @buffer: a #GstBuffer
+ * @format: GstFormat of @start and @stop, GST_FORMAT_DEFAULT is samples
+ * @start: Amount of audio to clip from start of buffer
+ * @end: Amount of  to clip from end of buffer
+ *
+ * Attaches #GstAudioClippingMeta metadata to @buffer with the given parameters.
+ *
+ * Returns: (transfer none): the #GstAudioClippingMeta on @buffer.
+ *
+ * Since: 1.8
+ */
+GstAudioClippingMeta *
+gst_buffer_add_audio_clipping_meta (GstBuffer * buffer,
+    GstFormat format, guint64 start, guint64 end)
+{
+  GstAudioClippingMeta *meta;
+
+  g_return_val_if_fail (format != GST_FORMAT_UNDEFINED, NULL);
+
+  meta =
+      (GstAudioClippingMeta *) gst_buffer_add_meta (buffer,
+      GST_AUDIO_CLIPPING_META_INFO, NULL);
+
+  meta->format = format;
+  meta->start = start;
+  meta->end = end;
+
+  return meta;
+}
+
+GType
+gst_audio_clipping_meta_api_get_type (void)
+{
+  static volatile GType type;
+  static const gchar *tags[] =
+      { GST_META_TAG_AUDIO_STR, GST_META_TAG_AUDIO_RATE_STR, NULL };
+
+  if (g_once_init_enter (&type)) {
+    GType _type = gst_meta_api_type_register ("GstAudioClippingMetaAPI", tags);
+    g_once_init_leave (&type, _type);
+  }
+  return type;
+}
+
+const GstMetaInfo *
+gst_audio_clipping_meta_get_info (void)
+{
+  static const GstMetaInfo *audio_clipping_meta_info = NULL;
+
+  if (g_once_init_enter ((GstMetaInfo **) & audio_clipping_meta_info)) {
+    const GstMetaInfo *meta =
+        gst_meta_register (GST_AUDIO_CLIPPING_META_API_TYPE,
+        "GstAudioClippingMeta", sizeof (GstAudioClippingMeta),
+        gst_audio_clipping_meta_init, NULL,
+        gst_audio_clipping_meta_transform);
+    g_once_init_leave ((GstMetaInfo **) & audio_clipping_meta_info,
+        (GstMetaInfo *) meta);
+  }
+  return audio_clipping_meta_info;
+}
