@@ -260,7 +260,8 @@ gst_video_overlay_composition_meta_get_info (void)
 {
   static const GstMetaInfo *video_overlay_composition_meta_info = NULL;
 
-  if (g_once_init_enter (&video_overlay_composition_meta_info)) {
+  if (g_once_init_enter ((GstMetaInfo **) &
+          video_overlay_composition_meta_info)) {
     const GstMetaInfo *meta =
         gst_meta_register (GST_VIDEO_OVERLAY_COMPOSITION_META_API_TYPE,
         "GstVideoOverlayCompositionMeta",
@@ -269,7 +270,8 @@ gst_video_overlay_composition_meta_get_info (void)
         (GstMetaFreeFunction) gst_video_overlay_composition_meta_free,
         (GstMetaTransformFunction)
         gst_video_overlay_composition_meta_transform);
-    g_once_init_leave (&video_overlay_composition_meta_info, meta);
+    g_once_init_leave ((GstMetaInfo **) & video_overlay_composition_meta_info,
+        (GstMetaInfo *) meta);
   }
   return video_overlay_composition_meta_info;
 }
@@ -454,11 +456,15 @@ gst_video_overlay_rectangle_needs_scaling (GstVideoOverlayRectangle * r)
 /**
  * gst_video_overlay_composition_blend:
  * @comp: a #GstVideoOverlayComposition
- * @video_buf: a #GstVideoFrame containing raw video data in a supported format
+ * @video_buf: a #GstVideoFrame containing raw video data in a
+ *             supported format. It should be mapped using GST_MAP_READWRITE
  *
  * Blends the overlay rectangles in @comp on top of the raw video data
  * contained in @video_buf. The data in @video_buf must be writable and
  * mapped appropriately.
+ *
+ * Since @video_buf data is read and will be modified, it ought be
+ * mapped with flag GST_MAP_READWRITE.
  */
 /* FIXME: formats with more than 8 bit per component which get unpacked into
  * ARGB64 or AYUV64 (such as v210, v216, UYVP, GRAY16_LE and GRAY16_BE)
@@ -733,7 +739,10 @@ gst_video_overlay_rectangle_new_raw (GstBuffer * pixels,
   rect->scaled_rectangles = NULL;
 
   gst_video_info_init (&rect->info);
-  gst_video_info_set_format (&rect->info, format, width, height);
+  if (!gst_video_info_set_format (&rect->info, format, width, height)) {
+    gst_mini_object_unref (GST_MINI_OBJECT_CAST (rect));
+    return NULL;
+  }
   if (flags & GST_VIDEO_OVERLAY_FORMAT_FLAG_PREMULTIPLIED_ALPHA)
     rect->info.flags |= GST_VIDEO_FLAG_PREMULTIPLIED_ALPHA;
 
@@ -1053,7 +1062,10 @@ gst_video_overlay_rectangle_convert (GstVideoInfo * src, GstBuffer * src_buffer,
   height = GST_VIDEO_INFO_HEIGHT (src);
 
   gst_video_info_init (dest);
-  gst_video_info_set_format (dest, dest_format, width, height);
+  if (!gst_video_info_set_format (dest, dest_format, width, height)) {
+    g_warn_if_reached ();
+    return;
+  }
 
   *dest_buffer = gst_buffer_new_and_alloc (GST_VIDEO_INFO_SIZE (dest));
 
