@@ -29,14 +29,16 @@
  * The adder currently mixes all data received on the sinkpads as soon as
  * possible without trying to synchronize the streams.
  *
+ * Check out the audiomixer element in gst-plugins-bad for a better-behaving
+ * audio mixing element: It will sync input streams correctly and also handle
+ * live inputs properly.
+ *
  * <refsect2>
  * <title>Example launch line</title>
  * |[
- * gst-launch audiotestsrc freq=100 ! adder name=mix ! audioconvert ! alsasink audiotestsrc freq=500 ! mix.
+ * gst-launch-1.0 audiotestsrc freq=100 ! adder name=mix ! audioconvert ! autoaudiosink audiotestsrc freq=500 ! mix.
  * ]| This pipeline produces two sine waves mixed together.
  * </refsect2>
- *
- * Last reviewed on 2006-05-09 (0.10.7)
  */
 /* Element-Checklist-Version: 5 */
 
@@ -223,6 +225,8 @@ gst_adder_sink_getcaps (GstPad * pad, GstCaps * filter)
 {
   GstAdder *adder;
   GstCaps *result, *peercaps, *current_caps, *filter_caps;
+  GstStructure *s;
+  gint i, n;
 
   adder = GST_ADDER (GST_PAD_PARENT (pad));
 
@@ -283,6 +287,22 @@ gst_adder_sink_getcaps (GstPad * pad, GstCaps * filter)
     }
   }
 
+  result = gst_caps_make_writable (result);
+
+  n = gst_caps_get_size (result);
+  for (i = 0; i < n; i++) {
+    GstStructure *sref;
+
+    s = gst_caps_get_structure (result, i);
+    sref = gst_structure_copy (s);
+    gst_structure_set (sref, "channels", GST_TYPE_INT_RANGE, 0, 2, NULL);
+    if (gst_structure_is_subset (s, sref)) {
+      /* This field is irrelevant when in mono or stereo */
+      gst_structure_remove_field (s, "channel-mask");
+    }
+    gst_structure_free (sref);
+  }
+
   if (filter_caps)
     gst_caps_unref (filter_caps);
 
@@ -322,9 +342,19 @@ gst_adder_sink_query (GstCollectPads * pads, GstCollectData * pad,
  * the other sinkpads because we can only mix streams with the same caps.
  */
 static gboolean
-gst_adder_setcaps (GstAdder * adder, GstPad * pad, GstCaps * caps)
+gst_adder_setcaps (GstAdder * adder, GstPad * pad, GstCaps * orig_caps)
 {
+  GstCaps *caps;
   GstAudioInfo info;
+  GstStructure *s;
+  gint channels;
+
+  caps = gst_caps_copy (orig_caps);
+
+  s = gst_caps_get_structure (caps, 0);
+  if (gst_structure_get_int (s, "channels", &channels))
+    if (channels <= 2)
+      gst_structure_remove_field (s, "channel-mask");
 
   if (!gst_audio_info_from_caps (&info, caps))
     goto invalid_format;
@@ -337,12 +367,14 @@ gst_adder_setcaps (GstAdder * adder, GstPad * pad, GstCaps * caps)
   if (adder->current_caps != NULL) {
     if (gst_audio_info_is_equal (&info, &adder->info)) {
       GST_OBJECT_UNLOCK (adder);
+      gst_caps_unref (caps);
       return TRUE;
     } else {
       GST_DEBUG_OBJECT (pad, "got input caps %" GST_PTR_FORMAT ", but "
           "current caps are %" GST_PTR_FORMAT, caps, adder->current_caps);
       GST_OBJECT_UNLOCK (adder);
       gst_pad_push_event (pad, gst_event_new_reconfigure ());
+      gst_caps_unref (caps);
       return FALSE;
     }
   }
@@ -356,11 +388,14 @@ gst_adder_setcaps (GstAdder * adder, GstPad * pad, GstCaps * caps)
 
   GST_INFO_OBJECT (pad, "handle caps change to %" GST_PTR_FORMAT, caps);
 
+  gst_caps_unref (caps);
+
   return TRUE;
 
   /* ERRORS */
 invalid_format:
   {
+    gst_caps_unref (caps);
     GST_WARNING_OBJECT (adder, "invalid format set as caps");
     return FALSE;
   }
@@ -453,91 +488,6 @@ gst_adder_query_duration (GstAdder * adder, GstQuery * query)
   return res;
 }
 
-static gboolean
-gst_adder_query_latency (GstAdder * adder, GstQuery * query)
-{
-  GstClockTime min, max;
-  gboolean live;
-  gboolean res;
-  GstIterator *it;
-  gboolean done;
-  GValue item = { 0, };
-
-  res = TRUE;
-  done = FALSE;
-
-  live = FALSE;
-  min = 0;
-  max = GST_CLOCK_TIME_NONE;
-
-  /* Take maximum of all latency values */
-  it = gst_element_iterate_sink_pads (GST_ELEMENT_CAST (adder));
-  while (!done) {
-    GstIteratorResult ires;
-
-    ires = gst_iterator_next (it, &item);
-    switch (ires) {
-      case GST_ITERATOR_DONE:
-        done = TRUE;
-        break;
-      case GST_ITERATOR_OK:
-      {
-        GstPad *pad = g_value_get_object (&item);
-        GstQuery *peerquery;
-        GstClockTime min_cur, max_cur;
-        gboolean live_cur;
-
-        peerquery = gst_query_new_latency ();
-
-        /* Ask peer for latency */
-        res &= gst_pad_peer_query (pad, peerquery);
-
-        /* take max from all valid return values */
-        if (res) {
-          gst_query_parse_latency (peerquery, &live_cur, &min_cur, &max_cur);
-
-          if (min_cur > min)
-            min = min_cur;
-
-          if (max_cur != GST_CLOCK_TIME_NONE &&
-              ((max != GST_CLOCK_TIME_NONE && max_cur > max) ||
-                  (max == GST_CLOCK_TIME_NONE)))
-            max = max_cur;
-
-          live = live || live_cur;
-        }
-
-        gst_query_unref (peerquery);
-        g_value_reset (&item);
-        break;
-      }
-      case GST_ITERATOR_RESYNC:
-        live = FALSE;
-        min = 0;
-        max = GST_CLOCK_TIME_NONE;
-        res = TRUE;
-        gst_iterator_resync (it);
-        break;
-      default:
-        res = FALSE;
-        done = TRUE;
-        break;
-    }
-  }
-  g_value_unset (&item);
-  gst_iterator_free (it);
-
-  if (res) {
-    /* store the results */
-    GST_DEBUG_OBJECT (adder, "Calculated total latency: live %s, min %"
-        GST_TIME_FORMAT ", max %" GST_TIME_FORMAT,
-        (live ? "yes" : "no"), GST_TIME_ARGS (min), GST_TIME_ARGS (max));
-    gst_query_set_latency (query, live, min, max);
-  }
-
-  return res;
-}
-
 static gboolean
 gst_adder_src_query (GstPad * pad, GstObject * parent, GstQuery * query)
 {
@@ -569,9 +519,6 @@ gst_adder_src_query (GstPad * pad, GstObject * parent, GstQuery * query)
     case GST_QUERY_DURATION:
       res = gst_adder_query_duration (adder, query);
       break;
-    case GST_QUERY_LATENCY:
-      res = gst_adder_query_latency (adder, query);
-      break;
     default:
       /* FIXME, needs a custom query handler because we have multiple
        * sinkpads */
@@ -909,13 +856,12 @@ gst_adder_class_init (GstAdderClass * klass)
           "object.", GST_TYPE_CAPS,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_adder_src_template));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_adder_sink_template));
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_adder_src_template);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_adder_sink_template);
   gst_element_class_set_static_metadata (gstelement_class, "Adder",
-      "Generic/Audio",
-      "Add N audio channels together",
+      "Generic/Audio", "Add N audio channels together",
       "Thomas Vander Stichele <thomas at apestaart dot org>");
 
   gstelement_class->request_new_pad =
@@ -1143,6 +1089,8 @@ gst_adder_collected (GstCollectPads * pads, gpointer user_data)
   gint64 next_timestamp;
   gint rate, bps, bpf;
   gboolean had_mute = FALSE;
+  gboolean is_eos = TRUE;
+  gboolean is_discont = FALSE;
 
   adder = GST_ADDER (user_data);
 
@@ -1150,7 +1098,7 @@ gst_adder_collected (GstCollectPads * pads, gpointer user_data)
   if (G_UNLIKELY (adder->info.finfo->format == GST_AUDIO_FORMAT_UNKNOWN))
     goto not_negotiated;
 
-  if (adder->flush_stop_pending == TRUE) {
+  if (adder->flush_stop_pending) {
     GST_INFO_OBJECT (adder->srcpad, "send pending flush stop event");
     if (!gst_pad_push_event (adder->srcpad, gst_event_new_flush_stop (TRUE))) {
       GST_WARNING_OBJECT (adder->srcpad, "Sending flush stop event failed");
@@ -1161,11 +1109,17 @@ gst_adder_collected (GstCollectPads * pads, gpointer user_data)
 
   if (adder->send_stream_start) {
     gchar s_id[32];
+    GstEvent *event;
 
     GST_INFO_OBJECT (adder->srcpad, "send pending stream start event");
-    /* stream-start (FIXME: create id based on input ids) */
+    /* FIXME: create id based on input ids, we can't use
+     * gst_pad_create_stream_id() though as that only handles 0..1 sink-pad
+     */
     g_snprintf (s_id, sizeof (s_id), "adder-%08x", g_random_int ());
-    if (!gst_pad_push_event (adder->srcpad, gst_event_new_stream_start (s_id))) {
+    event = gst_event_new_stream_start (s_id);
+    gst_event_set_group_id (event, gst_util_group_id_next ());
+
+    if (!gst_pad_push_event (adder->srcpad, event)) {
       GST_WARNING_OBJECT (adder->srcpad, "Sending stream start event failed");
     }
     adder->send_stream_start = FALSE;
@@ -1183,17 +1137,46 @@ gst_adder_collected (GstCollectPads * pads, gpointer user_data)
     adder->send_caps = FALSE;
   }
 
-  /* get available bytes for reading, this can be 0 which could mean empty
-   * buffers or EOS, which we will catch when we loop over the pads. */
-  outsize = gst_collect_pads_available (pads);
-  /* can only happen when no pads to collect or all EOS */
-  if (outsize == 0)
-    goto eos;
-
   rate = GST_AUDIO_INFO_RATE (&adder->info);
   bps = GST_AUDIO_INFO_BPS (&adder->info);
   bpf = GST_AUDIO_INFO_BPF (&adder->info);
 
+  if (g_atomic_int_compare_and_exchange (&adder->new_segment_pending, TRUE,
+          FALSE)) {
+    GstEvent *event;
+
+    /*
+     * When seeking we set the start and stop positions as given in the seek
+     * event. We also adjust offset & timestamp accordingly.
+     * This basically ignores all newsegments sent by upstream.
+     */
+    event = gst_event_new_segment (&adder->segment);
+    if (adder->segment.rate > 0.0) {
+      adder->segment.position = adder->segment.start;
+    } else {
+      adder->segment.position = adder->segment.stop;
+    }
+    adder->offset = gst_util_uint64_scale (adder->segment.position,
+        rate, GST_SECOND);
+
+    GST_INFO_OBJECT (adder->srcpad, "sending pending new segment event %"
+        GST_SEGMENT_FORMAT, &adder->segment);
+    if (event) {
+      if (!gst_pad_push_event (adder->srcpad, event)) {
+        GST_WARNING_OBJECT (adder->srcpad, "Sending new segment event failed");
+      }
+    } else {
+      GST_WARNING_OBJECT (adder->srcpad, "Creating new segment event for "
+          "start:%" G_GINT64_FORMAT "  end:%" G_GINT64_FORMAT " failed",
+          adder->segment.start, adder->segment.stop);
+    }
+    is_discont = TRUE;
+  }
+
+  /* get available bytes for reading, this can be 0 which could mean empty
+   * buffers or EOS, which we will catch when we loop over the pads. */
+  outsize = gst_collect_pads_available (pads);
+
   GST_LOG_OBJECT (adder,
       "starting to cycle through channels, %d bytes available (bps = %d, bpf = %d)",
       outsize, bps, bpf);
@@ -1214,6 +1197,11 @@ gst_adder_collected (GstCollectPads * pads, gpointer user_data)
     /* get a buffer of size bytes, if we get a buffer, it is at least outsize
      * bytes big. */
     inbuf = gst_collect_pads_take_buffer (pads, collect_data, outsize);
+
+    if (!GST_COLLECT_PADS_STATE_IS_SET (collect_data,
+            GST_COLLECT_PADS_STATE_EOS))
+      is_eos = FALSE;
+
     /* NULL means EOS or an empty buffer so we still need to flush in
      * case of an empty buffer. */
     if (inbuf == NULL) {
@@ -1407,9 +1395,13 @@ gst_adder_collected (GstCollectPads * pads, gpointer user_data)
     }
     GST_OBJECT_UNLOCK (pad);
   }
+
   if (outbuf)
     gst_buffer_unmap (outbuf, &outmap);
 
+  if (is_eos)
+    goto eos;
+
   if (outbuf == NULL) {
     /* no output buffer, reuse one of the GAP buffers then if we have one */
     if (gapbuf) {
@@ -1433,37 +1425,6 @@ gst_adder_collected (GstCollectPads * pads, gpointer user_data)
     gst_buffer_unref (gapbuf);
   }
 
-  if (g_atomic_int_compare_and_exchange (&adder->new_segment_pending, TRUE,
-          FALSE)) {
-    GstEvent *event;
-
-    /* 
-     * When seeking we set the start and stop positions as given in the seek
-     * event. We also adjust offset & timestamp accordingly.
-     * This basically ignores all newsegments sent by upstream.
-     */
-    event = gst_event_new_segment (&adder->segment);
-    if (adder->segment.rate > 0.0) {
-      adder->segment.position = adder->segment.start;
-    } else {
-      adder->segment.position = adder->segment.stop;
-    }
-    adder->offset = gst_util_uint64_scale (adder->segment.position,
-        rate, GST_SECOND);
-
-    GST_INFO_OBJECT (adder->srcpad, "sending pending new segment event %"
-        GST_SEGMENT_FORMAT, &adder->segment);
-    if (event) {
-      if (!gst_pad_push_event (adder->srcpad, event)) {
-        GST_WARNING_OBJECT (adder->srcpad, "Sending new segment event failed");
-      }
-    } else {
-      GST_WARNING_OBJECT (adder->srcpad, "Creating new segment event for "
-          "start:%" G_GINT64_FORMAT "  end:%" G_GINT64_FORMAT " failed",
-          adder->segment.start, adder->segment.stop);
-    }
-  }
-
   if (G_UNLIKELY (adder->pending_events)) {
     GList *tmp = adder->pending_events;
 
@@ -1500,6 +1461,11 @@ gst_adder_collected (GstCollectPads * pads, gpointer user_data)
     GST_BUFFER_OFFSET_END (outbuf) = adder->offset;
     GST_BUFFER_DURATION (outbuf) = adder->segment.position - next_timestamp;
   }
+  if (is_discont) {
+    GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
+  } else {
+    GST_BUFFER_FLAG_UNSET (outbuf, GST_BUFFER_FLAG_DISCONT);
+  }
 
   adder->offset = next_offset;
   adder->segment.position = next_timestamp;
