@@ -625,7 +625,7 @@ gst_decodebin3_dispose (GObject * object)
     g_list_free (dbin->decoder_factories);
   if (dbin->decodable_factories)
     g_list_free (dbin->decodable_factories);
-  g_list_free (dbin->requested_selection);
+  g_list_free_full (dbin->requested_selection, g_free);
   g_list_free (dbin->active_selection);
   g_list_free (dbin->to_activate);
   g_list_free (dbin->pending_select_streams);
@@ -998,7 +998,7 @@ gst_decode_bin_update_factories_list (GstDecodebin3 * dbin)
 }
 
 /* Must be called with appropriate lock if list is a protected variable */
-static gboolean
+static const gchar *
 stream_in_list (GList * list, const gchar * sid)
 {
   GList *tmp;
@@ -1011,12 +1011,12 @@ stream_in_list (GList * list, const gchar * sid)
 #endif
 
   for (tmp = list; tmp; tmp = tmp->next) {
-    gchar *osid = (gchar *) tmp->data;
+    const gchar *osid = (gchar *) tmp->data;
     if (!g_strcmp0 (sid, osid))
-      return TRUE;
+      return osid;
   }
 
-  return FALSE;
+  return NULL;
 }
 
 static void
@@ -1088,10 +1088,12 @@ beach:
     if (dbin->requested_selection) {
       GST_FIXME_OBJECT (dbin,
           "Replacing non-NULL requested_selection, what should we do ??");
-      g_list_free (dbin->requested_selection);
+      g_list_free_full (dbin->requested_selection, g_free);
     }
-    dbin->requested_selection = tmp;
+    dbin->requested_selection =
+        g_list_copy_deep (tmp, (GCopyFunc) g_strdup, NULL);
     dbin->selection_updated = TRUE;
+    g_list_free (tmp);
   }
   SELECTION_UNLOCK (dbin);
 }
@@ -1192,22 +1194,22 @@ find_message_parsebin (GstDecodebin3 * dbin, GstElement * child)
   return input;
 }
 
-static gboolean
+static const gchar *
 stream_in_collection (GstDecodebin3 * dbin, gchar * sid)
 {
   guint i, len;
 
   if (dbin->collection == NULL)
-    return FALSE;
+    return NULL;
   len = gst_stream_collection_get_size (dbin->collection);
   for (i = 0; i < len; i++) {
     GstStream *stream = gst_stream_collection_get_stream (dbin->collection, i);
     const gchar *osid = gst_stream_get_stream_id (stream);
     if (!g_strcmp0 (sid, osid))
-      return TRUE;
+      return osid;
   }
 
-  return FALSE;
+  return NULL;
 }
 
 /* Call with INPUT_LOCK taken */
@@ -1356,6 +1358,7 @@ get_output_for_slot (MultiQueueSlot * slot)
   DecodebinOutputStream *output = NULL;
   const gchar *stream_id;
   GstCaps *caps;
+  gchar *id_in_list = NULL;
 
   /* If we already have a configured output, just use it */
   if (slot->output != NULL)
@@ -1406,7 +1409,8 @@ get_output_for_slot (MultiQueueSlot * slot)
 #endif
 
   /* 3. In default mode check if we should expose */
-  if (stream_in_list (dbin->requested_selection, stream_id)) {
+  id_in_list = (gchar *) stream_in_list (dbin->requested_selection, stream_id);
+  if (id_in_list) {
     /* Check if we can steal an existing output stream we could re-use.
      * that is:
      * * an output stream whose slot->stream is not in requested
@@ -1418,7 +1422,8 @@ get_output_for_slot (MultiQueueSlot * slot)
       dbin->to_activate =
           g_list_append (dbin->to_activate, (gchar *) stream_id);
       dbin->requested_selection =
-          g_list_remove (dbin->requested_selection, stream_id);
+          g_list_remove (dbin->requested_selection, id_in_list);
+      g_free (id_in_list);
       SELECTION_UNLOCK (dbin);
       gst_pad_add_probe (output->slot->src_pad, GST_PAD_PROBE_TYPE_IDLE,
           (GstPadProbeCallback) slot_unassign_probe, output->slot, NULL);
@@ -1587,15 +1592,17 @@ multiqueue_src_probe (GstPad * pad, GstPadProbeInfo * info,
             dbin->output_streams = g_list_remove (dbin->output_streams, output);
             free_output_stream (dbin, output);
           }
+          slot->probe_id = 0;
           dbin->slots = g_list_remove (dbin->slots, slot);
           free_multiqueue_slot_async (dbin, slot);
           SELECTION_UNLOCK (dbin);
-          ret = GST_PAD_PROBE_HANDLED;
+          ret = GST_PAD_PROBE_REMOVE;
         }
         break;
       case GST_EVENT_CUSTOM_DOWNSTREAM:
         if (gst_event_has_name (ev, "decodebin3-custom-eos")) {
           slot->is_drained = TRUE;
+          ret = GST_PAD_PROBE_DROP;
           SELECTION_LOCK (dbin);
           if (slot->input == NULL) {
             GST_DEBUG_OBJECT (pad,
@@ -1607,11 +1614,12 @@ multiqueue_src_probe (GstPad * pad, GstPadProbeInfo * info,
                   g_list_remove (dbin->output_streams, output);
               free_output_stream (dbin, output);
             }
+            slot->probe_id = 0;
             dbin->slots = g_list_remove (dbin->slots, slot);
             free_multiqueue_slot_async (dbin, slot);
+            ret = GST_PAD_PROBE_REMOVE;
           }
           SELECTION_UNLOCK (dbin);
-          ret = GST_PAD_PROBE_DROP;
         }
         break;
       default:
@@ -2130,7 +2138,7 @@ reassign_slot (GstDecodebin3 * dbin, MultiQueueSlot * slot)
       tsid = tmp->data;
       /* Pass target stream id to requested selection */
       dbin->requested_selection =
-          g_list_append (dbin->requested_selection, tmp->data);
+          g_list_append (dbin->requested_selection, g_strdup (tmp->data));
       dbin->to_activate = g_list_remove (dbin->to_activate, tmp->data);
       break;
     }
@@ -2328,27 +2336,33 @@ handle_stream_switch (GstDecodebin3 * dbin, GList * select_streams,
   if (to_activate == NULL && pending_streams != NULL) {
     GST_DEBUG_OBJECT (dbin, "Stream switch requested for future collection");
     if (dbin->requested_selection)
-      g_list_free (dbin->requested_selection);
-    dbin->requested_selection = select_streams;
+      g_list_free_full (dbin->requested_selection, g_free);
+    dbin->requested_selection =
+        g_list_copy_deep (select_streams, (GCopyFunc) g_strdup, NULL);
     g_list_free (to_deactivate);
     g_list_free (pending_streams);
     to_deactivate = NULL;
+    pending_streams = NULL;
   } else {
     if (dbin->requested_selection)
-      g_list_free (dbin->requested_selection);
-    dbin->requested_selection = future_request_streams;
+      g_list_free_full (dbin->requested_selection, g_free);
     dbin->requested_selection =
-        g_list_concat (dbin->requested_selection, pending_streams);
+        g_list_copy_deep (future_request_streams, (GCopyFunc) g_strdup, NULL);
+    dbin->requested_selection =
+        g_list_concat (dbin->requested_selection,
+        g_list_copy_deep (pending_streams, (GCopyFunc) g_strdup, NULL));
     if (dbin->to_activate)
       g_list_free (dbin->to_activate);
-    dbin->to_activate = to_reassign;
+    dbin->to_activate = g_list_copy (to_reassign);
   }
 
   dbin->selection_updated = TRUE;
   SELECTION_UNLOCK (dbin);
 
-  if (unknown)
+  if (unknown) {
     GST_FIXME_OBJECT (dbin, "Got request for an unknown stream");
+    g_list_free (unknown);
+  }
 
   /* For all streams to deactivate, add an idle probe where we will do
    * the unassignment and switch over */
@@ -2358,6 +2372,19 @@ handle_stream_switch (GstDecodebin3 * dbin, GList * select_streams,
         (GstPadProbeCallback) slot_unassign_probe, slot, NULL);
   }
 
+  if (to_deactivate)
+    g_list_free (to_deactivate);
+  if (to_activate)
+    g_list_free (to_activate);
+  if (to_reassign)
+    g_list_free (to_reassign);
+  if (future_request_streams)
+    g_list_free (future_request_streams);
+  if (pending_streams)
+    g_list_free (pending_streams);
+  if (slots_to_reassign)
+    g_list_free (slots_to_reassign);
+
   return ret;
 }
 
@@ -2403,8 +2430,10 @@ ghost_pad_event_probe (GstPad * pad, GstPadProbeInfo * info,
         gst_event_unref (event);
       }
       /* Finally handle the switch */
-      if (streams)
+      if (streams) {
         handle_stream_switch (dbin, streams, seqnum);
+        g_list_free_full (streams, g_free);
+      }
       ret = GST_PAD_PROBE_HANDLED;
     }
       break;
@@ -2450,8 +2479,10 @@ gst_decodebin3_send_event (GstElement * element, GstEvent * event)
     }
 #endif
     /* Finally handle the switch */
-    if (streams)
+    if (streams) {
       handle_stream_switch (dbin, streams, seqnum);
+      g_list_free_full (streams, g_free);
+    }
 
     gst_event_unref (event);
     return TRUE;
