@@ -49,7 +49,7 @@
 
 GST_DEBUG_CATEGORY (videoconvert_debug);
 #define GST_CAT_DEFAULT videoconvert_debug
-GST_DEBUG_CATEGORY_EXTERN (GST_CAT_PERFORMANCE);
+GST_DEBUG_CATEGORY_STATIC (CAT_PERFORMANCE);
 
 GType gst_video_convert_get_type (void);
 
@@ -67,6 +67,7 @@ G_DEFINE_TYPE (GstVideoConvert, gst_video_convert, GST_TYPE_VIDEO_FILTER);
 #define DEFAULT_PROP_MATRIX_MODE GST_VIDEO_MATRIX_MODE_FULL
 #define DEFAULT_PROP_GAMMA_MODE GST_VIDEO_GAMMA_MODE_NONE
 #define DEFAULT_PROP_PRIMARIES_MODE GST_VIDEO_PRIMARIES_MODE_NONE
+#define DEFAULT_PROP_N_THREADS 1
 
 enum
 {
@@ -79,7 +80,8 @@ enum
   PROP_CHROMA_MODE,
   PROP_MATRIX_MODE,
   PROP_GAMMA_MODE,
-  PROP_PRIMARIES_MODE
+  PROP_PRIMARIES_MODE,
+  PROP_N_THREADS
 };
 
 #define CSP_VIDEO_CAPS GST_VIDEO_CAPS_MAKE (GST_VIDEO_FORMATS_ALL) ";" \
@@ -464,7 +466,9 @@ gst_video_convert_set_info (GstVideoFilter * filter,
           GST_VIDEO_CONVERTER_OPT_GAMMA_MODE,
           GST_TYPE_VIDEO_GAMMA_MODE, space->gamma_mode,
           GST_VIDEO_CONVERTER_OPT_PRIMARIES_MODE,
-          GST_TYPE_VIDEO_PRIMARIES_MODE, space->primaries_mode, NULL));
+          GST_TYPE_VIDEO_PRIMARIES_MODE, space->primaries_mode,
+          GST_VIDEO_CONVERTER_OPT_THREADS, G_TYPE_UINT,
+          space->n_threads, NULL));
   if (space->convert == NULL)
     goto no_convert;
 
@@ -511,15 +515,15 @@ gst_video_convert_class_init (GstVideoConvertClass * klass)
   gobject_class->get_property = gst_video_convert_get_property;
   gobject_class->finalize = gst_video_convert_finalize;
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_video_convert_src_template));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&gst_video_convert_sink_template));
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_video_convert_src_template);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &gst_video_convert_sink_template);
 
   gst_element_class_set_static_metadata (gstelement_class,
       "Colorspace converter", "Filter/Converter/Video",
       "Converts video from one colorspace to another",
-      "GStreamer maintainers <gstreamer-devel@lists.sourceforge.net>");
+      "GStreamer maintainers <gstreamer-devel@lists.freedesktop.org>");
 
   gstbasetransform_class->transform_caps =
       GST_DEBUG_FUNCPTR (gst_video_convert_transform_caps);
@@ -576,6 +580,10 @@ gst_video_convert_class_init (GstVideoConvertClass * klass)
           "Primaries Conversion Mode", gst_video_primaries_mode_get_type (),
           DEFAULT_PROP_PRIMARIES_MODE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (gobject_class, PROP_N_THREADS,
+      g_param_spec_uint ("n-threads", "Threads",
+          "Maximum number of threads to use", 0, G_MAXUINT,
+          DEFAULT_PROP_N_THREADS, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -590,6 +598,7 @@ gst_video_convert_init (GstVideoConvert * space)
   space->matrix_mode = DEFAULT_PROP_MATRIX_MODE;
   space->gamma_mode = DEFAULT_PROP_GAMMA_MODE;
   space->primaries_mode = DEFAULT_PROP_PRIMARIES_MODE;
+  space->n_threads = DEFAULT_PROP_N_THREADS;
 }
 
 void
@@ -628,6 +637,9 @@ gst_video_convert_set_property (GObject * object, guint property_id,
     case PROP_DITHER_QUANTIZATION:
       csp->dither_quantization = g_value_get_uint (value);
       break;
+    case PROP_N_THREADS:
+      csp->n_threads = g_value_get_uint (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -670,6 +682,9 @@ gst_video_convert_get_property (GObject * object, guint property_id,
     case PROP_DITHER_QUANTIZATION:
       g_value_set_uint (value, csp->dither_quantization);
       break;
+    case PROP_N_THREADS:
+      g_value_set_uint (value, csp->n_threads);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -684,7 +699,7 @@ gst_video_convert_transform_frame (GstVideoFilter * filter,
 
   space = GST_VIDEO_CONVERT_CAST (filter);
 
-  GST_CAT_DEBUG_OBJECT (GST_CAT_PERFORMANCE, filter,
+  GST_CAT_DEBUG_OBJECT (CAT_PERFORMANCE, filter,
       "doing colorspace conversion from %s -> to %s",
       GST_VIDEO_INFO_NAME (&filter->in_info),
       GST_VIDEO_INFO_NAME (&filter->out_info));
@@ -700,6 +715,8 @@ plugin_init (GstPlugin * plugin)
   GST_DEBUG_CATEGORY_INIT (videoconvert_debug, "videoconvert", 0,
       "Colorspace Converter");
 
+  GST_DEBUG_CATEGORY_GET (CAT_PERFORMANCE, "GST_PERFORMANCE");
+
   _colorspace_quark = g_quark_from_static_string ("colorspace");
 
   return gst_element_register (plugin, "videoconvert",
